#include "display.h"

//Display-Buffer
uint8_t buffer[8];
volatile uint8_t bufferpos;

//Display-Maske
/*       ___1____
 *      /       /
 *     /2      /0
 *    /___3___/
 *   /       /
 *  /5    4 /  __7
 * /___6___/  /_/
 *
 */
 #ifdef CHARMASK
uint8_t charmask[8] PROGMEM = {1,0,4,7,5,2,3,6};
#endif
#ifdef SEGMASK
uint8_t segmask[8] PROGMEM  = {1,0,3,2,5,4,7,6};
#endif

inline void display_init_i(void)
{
    //Ports auf Ausgnge setzen
    SEG_DRIVER_DDR = CHAR_DRIVER_DDR = 0xFF;
    //Prescaler setzen
    TCCR0B |= PRESCALER_VALUE;
    //Overflow-Interrupt aktivieren
    TIMSK0 |= (1 << TOIE0); 
    //Globale Interrupts aktivieren
    sei();
}

#ifdef DOMASK_INLINE
inline
#endif
char DoMask(char value)
{
    uint8_t i = 7;
    char end = 0;
    do
        if(value & (1 << i))
            end |= 1 << pgm_read_byte(&charmask[i]);
    while(i--); 
    return end;
}

inline void display_putc_i(char c, uint8_t pos)
{
    #ifdef CHARMASK
    c = DoMask(c);
    #endif
    #ifdef SEGMASK
    buffer[pgm_read_byte(&segmask[pos])] = c;
    #else
    buffer[pos] = c;
    #endif
}

inline void display_putsx_i(char* c, uint8_t pos)
{
    while(*c != '\0')
        display_putc(*c++,pos++);
}

inline inline void display_puts_i(char* c)
{
    display_putsx(c, 0);   
}

inline void display_clear_to_i(uint8_t value)
{
    uint8_t pos = 7;
    do
        buffer[pos] = value; 
    while(pos--);
}

inline void display_clear_i(void)
{
    display_clear_to(0x00);
}

inline void display_test_i(void)
{
    display_clear_to(0xFF);
}

#ifdef USE_FONT

inline void display_putcf_i(char c, uint8_t pos)
{
    display_putc(FONT_FUNCTION(c), pos);
}

inline void display_putsf_i(char* c)
{
    display_putsxf(c, 0);  
}

inline void display_putsxf_i(char* c, uint8_t pos)
{
    while(*c != '\0')
        display_putcf(*c++,pos++);
}
#endif


#ifndef TIMER0_OVF_vect
// Fr ltere WinAVR Versionen z.B. WinAVR-20071221 
#define TIMER0_OVF_vect TIMER0_OVF0_vect
#endif
 
ISR (TIMER0_OVF_vect)
{
    #ifdef SAVEMODE
    SEG_DRIVER_PORT = 0x00;
    #endif
    CHAR_DRIVER_PORT = ~(1 << bufferpos);
    SEG_DRIVER_PORT = buffer[bufferpos];
    if(++bufferpos == 8)
        bufferpos = 0;
}

//Keine Inline Funktionen

void display_init(void)
{
    display_init_i();   
}
void display_putc(char c, uint8_t pos)
{
    display_putc_i(c,pos);
}
void display_putsx(char* c, uint8_t pos)
{
    display_putsx_i(c,pos);
}
void display_puts(char* c)
{
    display_puts_i(c);
}
void display_putcf(char c, uint8_t pos)
{
    display_putcf_i(c,pos);
}
void display_putsxf(char* c, uint8_t pos)
{
    display_putsxf_i(c,pos);
}
void display_putsf(char* c)
{
    display_putsf_i(c);
}
